package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kstarbound.defs.TileDefinition
import ru.dbotthepony.kstarbound.math.Vector2i

class World(val seed: Long = 0L) {
	private val chunkMap = ArrayList<Pair<ChunkPos, Chunk>>()
	private var lastAccessedChunk: Chunk? = null

	fun getChunk(pos: ChunkPos): Chunk? {
		if (lastAccessedChunk?.pos == pos) {
			return lastAccessedChunk
		}

		for ((k, v) in chunkMap) {
			if (k == pos) {
				lastAccessedChunk = v
				return v
			}
		}

		return null
	}

	fun getOrMakeChunk(pos: ChunkPos): Chunk {
		if (lastAccessedChunk?.pos == pos) {
			return lastAccessedChunk!!
		}

		for ((k, v) in chunkMap) {
			if (k == pos) {
				return v
			}
		}

		val chunk = Chunk(this, pos)
		lastAccessedChunk = chunk
		chunkMap.add(pos to chunk)
		return chunk
	}

	fun getChunk(pos: Vector2i) = getChunk(ChunkPos(pos))
	fun getOrMakeChunk(pos: Vector2i) = getOrMakeChunk(ChunkPos(pos))

	fun getTile(pos: Vector2i): ChunkTile? {
		return getChunk(pos)?.get(pos.x, pos.y)
	}

	fun setTile(pos: Vector2i, tile: TileDefinition?): Chunk {
		val chunk = getOrMakeChunk(pos)
		chunk[pos.x and CHUNK_SIZE_FF, pos.y and CHUNK_SIZE_FF] = tile
		return chunk
	}
}
